@@ -1,42 +1,25 @@
-use std::{
-	mem::MaybeUninit,
-	borrow::Borrow
+use crate::generic::{
+	map::{BTreeMap, M},
+	node::{Address, Balance, Item, Node, Offset},
 };
+use cc_traits::{Slab, SlabMut};
 use smallvec::SmallVec;
-use cc_traits::{
-	Slab,
-	SlabMut
-};
-use crate::{
-	generic::{
-		map::{
-			M,
-			BTreeMap
-		},
-		node::{
-			Node,
-			Balance,
-			Item,
-			Address,
-			Offset
-		}
-	}
-};
+use std::{borrow::Borrow, mem::MaybeUninit};
 
 /// Extended API.
 ///
 /// This trait can be imported to access the internal functions of the B-Tree.
 /// These functions are not intended to be directly called by the users, but can be used to
 /// extends the data structure with new functionalities.
-/// 
+///
 /// # Addressing
-/// 
+///
 /// In this implementation of B-Trees, each node of a tree is addressed
 /// by the [`Address`] type.
 /// Each node is identified by a `usize`, and each item/entry in the node by an [`Offset`].
 /// This extended API allows the caller to explore, access and modify the
 /// internal structure of the tree using this addressing system.
-/// 
+///
 /// Note that a valid address does not always refer to an actual item in the tree.
 /// See the [`Address`] type documentation for more details.
 pub trait BTreeExt<K, V> {
@@ -51,23 +34,26 @@ pub trait BTreeExt<K, V> {
 	fn node(&self, id: usize) -> &Node<K, V>;
 
 	/// Get a reference to the value associated to the given `key` in the node `id`, if any.
-	fn get_in<Q: ?Sized>(&self, key: &Q, id: usize) -> Option<&V> where K: Borrow<Q>, Q: Ord;
+	fn get_in<Q: ?Sized>(&self, key: &Q, id: usize) -> Option<&V>
+	where
+		K: Borrow<Q>,
+		Q: Ord;
 
 	/// Get a reference to the item located at the given address.
 	fn item(&self, addr: Address) -> Option<&Item<K, V>>;
 
 	/// Get the first item address, if any.
-	/// 
+	///
 	/// Returns the first occupied valid address, or `None` if the tree is empty.
 	fn first_item_address(&self) -> Option<Address>;
 
 	/// Get the first back address.
-	/// 
+	///
 	/// The returned address may not be occupied if the tree is empty.
 	fn first_back_address(&self) -> Address;
 
 	/// Get the last item address, if any.
-	/// 
+	///
 	/// Returns the last occupied valid address, or `None` if the tree is empty.
 	fn last_item_address(&self) -> Option<Address>;
 
@@ -78,25 +64,25 @@ pub trait BTreeExt<K, V> {
 	fn normalize(&self, addr: Address) -> Option<Address>;
 
 	/// Returns the greatest valid leaf address that directly precedes the given address.
-	/// 
+	///
 	/// A "leaf address" is an address located in a leaf node.
 	fn leaf_address(&self, addr: Address) -> Address;
 
 	/// Get the previous item address.
-	/// 
+	///
 	/// Returns the previous valid occupied address.
-	/// 
+	///
 	/// The following diagram shows the order between addresses defined by this function.
 	/// ```text
-	///                                          ┌───────────┐ 
-	///                            ╔═════════════╪══╗  ╔══╗  │ 
+	///                                          ┌───────────┐
+	///                            ╔═════════════╪══╗  ╔══╗  │
 	///                            ║             │┌─v─┐║┌─v─┐│  
 	///                ┌───────────╫─────────────││ 0 │║│ 1 ││──────────────────────┐
 	///                │           ║             │└─v─┘║└─v─┘│                      │
 	///                │           ║             └──╫──╫──╫──┘                      │
 	///    start v     │           ║                ║  ║│ ╚══════════════════════╗  │  ^ end
-	///          ║     │           ║             ╔══╝  ╚╪══════════╗             ║  │  ║ 
-	///       ┌──╫──────────────┐  ║          ┌──╫──────────────┐  ║          ┌──╫─────╫──┐ 
+	///          ║     │           ║             ╔══╝  ╚╪══════════╗             ║  │  ║
+	///       ┌──╫──────────────┐  ║          ┌──╫──────────────┐  ║          ┌──╫─────╫──┐
 	///       │  ║     ╔═════╗  │  ║          │  ║     ╔═════╗  │  ║          │  ║     ║  │
 	///       │┌─v─┐ ┌─^─┐ ┌─v─┐│  ║          │┌─v─┐ ┌─^─┐ ┌─v─┐│  ║          │┌─v─┐ ┌─^─┐│
 	///       ││ 0 │ │ 1 │ │ 2 ││  ║          ││ 0 │ │ 1 │ │ 2 ││  ║          ││ 0 │ │ 1 ││
@@ -107,10 +93,10 @@ pub trait BTreeExt<K, V> {
 	fn previous_item_address(&self, addr: Address) -> Option<Address>;
 
 	/// Get the previous front address.
-	/// 
+	///
 	/// A "front address" is a valid address whose offset is less that the number of items in the node.
 	/// If `addr.offset` is equal to `-1`, then it doesn't actually refer to an existing item in the node.
-	/// 
+	///
 	/// The following diagram shows the order between addresses defined by this function.
 	/// ```text
 	///                                                         ^ end
@@ -134,20 +120,20 @@ pub trait BTreeExt<K, V> {
 	fn previous_front_address(&self, addr: Address) -> Option<Address>;
 
 	/// Get the next item address.
-	/// 
+	///
 	/// Returns the next valid occupied address.
-	/// 
+	///
 	/// The following diagram shows the order between addresses defined by this function.
 	/// ```text
-	///                                          ┌───────────┐ 
-	///                            ╔═════════════╪══╗  ╔══╗  │ 
+	///                                          ┌───────────┐
+	///                            ╔═════════════╪══╗  ╔══╗  │
 	///                            ║             │┌─v─┐║┌─v─┐│  
 	///                ┌───────────╫─────────────││ 0 │║│ 1 ││──────────────────────┐
 	///                │           ║             │└─v─┘║└─v─┘│                      │
 	///                │           ║             └──╫──╫──╫──┘                      │
 	///    start v     │           ║                ║  ║│ ╚══════════════════════╗  │  ^ end
-	///          ║     │           ║             ╔══╝  ╚╪══════════╗             ║  │  ║ 
-	///       ┌──╫──────────────┐  ║          ┌──╫──────────────┐  ║          ┌──╫─────╫──┐ 
+	///          ║     │           ║             ╔══╝  ╚╪══════════╗             ║  │  ║
+	///       ┌──╫──────────────┐  ║          ┌──╫──────────────┐  ║          ┌──╫─────╫──┐
 	///       │  ║     ╔═════╗  │  ║          │  ║     ╔═════╗  │  ║          │  ║     ║  │
 	///       │┌─v─┐ ┌─^─┐ ┌─v─┐│  ║          │┌─v─┐ ┌─^─┐ ┌─v─┐│  ║          │┌─v─┐ ┌─^─┐│
 	///       ││ 0 │ │ 1 │ │ 2 ││  ║          ││ 0 │ │ 1 │ │ 2 ││  ║          ││ 0 │ │ 1 ││
@@ -158,12 +144,12 @@ pub trait BTreeExt<K, V> {
 	fn next_item_address(&self, addr: Address) -> Option<Address>;
 
 	/// Get the next back address.
-	/// 
+	///
 	/// A "back address" is a valid address whose offset is at least `0`.
 	/// If `addr.offset` is equal to the number of items in the node then it doesn't actually refer
 	/// to an existing item in the node,
 	/// but it can be used to insert a new item with `BTreeExt::insert_at`.
-	/// 
+	///
 	/// The following diagram shows the order between addresses defined by this function.
 	/// ```text
 	///                                          ┌───────────┐  ^ end
@@ -188,38 +174,54 @@ pub trait BTreeExt<K, V> {
 	fn next_item_or_back_address(&self, addr: Address) -> Option<Address>;
 
 	/// Get the address of the given key.
-	/// 
+	///
 	/// Returns `Ok(addr)` if the key is used in the tree.
 	/// If the key is not used in the tree then `Err(addr)` is returned,
 	/// where `addr` can be used to insert the missing key.
-	fn address_of<Q: ?Sized>(&self, key: &Q) -> Result<Address, Address> where K: Borrow<Q>, Q: Ord;
+	fn address_of<Q: ?Sized>(&self, key: &Q) -> Result<Address, Address>
+	where
+		K: Borrow<Q>,
+		Q: Ord;
 
 	/// Search for the address of the given key from the given node `id`.
-	/// 
+	///
 	/// Users should directly use [`BTreeExt::address_of`].
-	fn address_in<Q: ?Sized>(&self, id: usize, key: &Q) -> Result<Address, Address> where K: Borrow<Q>, Q: Ord;
+	fn address_in<Q: ?Sized>(&self, id: usize, key: &Q) -> Result<Address, Address>
+	where
+		K: Borrow<Q>,
+		Q: Ord;
 
 	/// Validate the tree.
 	///
 	/// Panics if the tree is not a valid B-Tree.
 	#[cfg(debug_assertions)]
-	fn validate(&self) where K: Ord;
+	fn validate(&self)
+	where
+		K: Ord;
 
 	/// Validate the given node and returns the depth of the node.
 	///
 	/// Panics if the tree is not a valid B-Tree.
 	#[cfg(debug_assertions)]
-	fn validate_node(&self, id: usize, parent: Option<usize>, min: Option<&K>, max: Option<&K>) -> usize where K: Ord;
+	fn validate_node(
+		&self,
+		id: usize,
+		parent: Option<usize>,
+		min: Option<&K>,
+		max: Option<&K>,
+	) -> usize
+	where
+		K: Ord;
 }
 
 /// Extended mutable API.
-/// 
+///
 /// This trait can be imported to access and modify the internal functions of the B-Tree.
 /// These functions are not intended to be directly called by the users, but can be used to
 /// extends the data structure with new functionalities.
-/// 
+///
 /// # Correctness
-/// 
+///
 /// The user of this trait is responsible to preserve the invariants of the data-structure.
 /// In particular, no item must be modified or inserted in a way that
 /// break the order between keys.
@@ -230,39 +232,46 @@ pub trait BTreeExtMut<K, V> {
 	/// Set the root node identifier.
 	fn set_root_id(&mut self, id: Option<usize>);
 
-	/// Get the node associated to the given `id` mutabily.
+	/// Get the node associated to the given `id` mutably.
 	///
 	/// Panics if `id` is out of bounds.
 	fn node_mut(&mut self, id: usize) -> &mut Node<K, V>;
 
 	/// Get a mutable reference to the value associated to the given `key` in the node `id`, if any.
-	fn get_mut_in(&mut self, key: &K, id: usize) -> Option<&mut V> where K: Ord;
+	fn get_mut_in(&mut self, key: &K, id: usize) -> Option<&mut V>
+	where
+		K: Ord;
 
 	/// Get a mutable reference to the item located at the given address.
 	fn item_mut(&mut self, addr: Address) -> Option<&mut Item<K, V>>;
 
 	/// Insert an item at the given address.
-	/// 
+	///
 	/// The address is first converted into a leaf address using [`BTreeExt::leaf_address`]
 	/// and the item inserted using [`BTreeExtMut::insert_exactly_at`].
 	fn insert_at(&mut self, addr: Address, item: Item<K, V>) -> Address;
 
 	/// Insert an item at the given address.
-	/// 
+	///
 	/// If the address refers to an internal node,
 	/// `opt_right_id` defines the identifier of the child node inserted on the right of the inserted item.
-	/// 
+	///
 	/// Returns the address of the inserted item in the tree
 	/// (it may differ from the input address if the tree is rebalanced).
-	/// 
+	///
 	/// # Correctness
-	/// 
+	///
 	/// It is assumed that it is btree-correct to insert the given item at the given address.
-	/// 
+	///
 	/// # Panic
-	/// 
+	///
 	/// This function panics if the address refers to an internal node and `opt_right_id` is `None`.
-	fn insert_exactly_at(&mut self, addr: Address, item: Item<K, V>, opt_right_id: Option<usize>) -> Address;
+	fn insert_exactly_at(
+		&mut self,
+		addr: Address,
+		item: Item<K, V>,
+		opt_right_id: Option<usize>,
+	) -> Address;
 
 	/// Replaces the key-value binding at the given address.
 	fn replace_at(&mut self, addr: Address, key: K, value: V) -> (K, V);
@@ -271,7 +280,7 @@ pub trait BTreeExtMut<K, V> {
 	fn replace_value_at(&mut self, addr: Address, value: V) -> V;
 
 	/// Removes the item at the given address, if any.
-	/// 
+	///
 	/// If an item is removed then
 	/// this function returns a pair where the first hand side is the removed item,
 	/// and the right hand side is the updated address where the item can be reinserted at.
@@ -281,10 +290,16 @@ pub trait BTreeExtMut<K, V> {
 	fn rebalance(&mut self, node_id: usize, addr: Address) -> Address;
 
 	/// Update a value in the given node `node_id`.
-	fn update_in<T, F>(&mut self, id: usize, key: K, action: F) -> T where K: Ord, F: FnOnce(Option<V>) -> (Option<V>, T);
+	fn update_in<T, F>(&mut self, id: usize, key: K, action: F) -> T
+	where
+		K: Ord,
+		F: FnOnce(Option<V>) -> (Option<V>, T);
 
 	/// Update a valud at the given address.
-	fn update_at<T, F>(&mut self, addr: Address, action: F) -> T where K: Ord, F: FnOnce(V) -> (Option<V>, T);
+	fn update_at<T, F>(&mut self, addr: Address, action: F) -> T
+	where
+		K: Ord,
+		F: FnOnce(V) -> (Option<V>, T);
 
 	/// Take the right-most leaf value in the given node.
 	///
@@ -299,7 +314,10 @@ pub trait BTreeExtMut<K, V> {
 	fn release_node(&mut self, id: usize) -> Node<K, V>;
 }
 
-impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
+impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C>
+where
+	for<'r> C::ItemRef<'r>: Into<&'r Node<K, V>>,
+{
 	#[inline]
 	fn root_id(&self) -> Option<usize> {
 		self.root
@@ -307,17 +325,19 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 
 	#[inline]
 	fn node(&self, id: usize) -> &Node<K, V> {
-		self.nodes.get(id).unwrap()
+		self.nodes.get(id).unwrap().into()
 	}
 
 	#[inline]
-	fn get_in<Q: ?Sized>(&self, key: &Q, mut id: usize) -> Option<&V> where K: Borrow<Q>, Q: Ord {
+	fn get_in<Q: ?Sized>(&self, key: &Q, mut id: usize) -> Option<&V>
+	where
+		K: Borrow<Q>,
+		Q: Ord,
+	{
 		loop {
 			match self.node(id).get(key) {
 				Ok(value_opt) => return value_opt,
-				Err(child_id) => {
-					id = child_id
-				}
+				Err(child_id) => id = child_id,
 			}
 		}
 	}
@@ -330,13 +350,11 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 		match self.root {
 			Some(mut id) => loop {
 				match self.node(id).child_id_opt(0) {
-					Some(child_id) => {
-						id = child_id
-					},
-					None => return Some(Address::new(id, 0.into()))
+					Some(child_id) => id = child_id,
+					None => return Some(Address::new(id, 0.into())),
 				}
 			},
-			None => None
+			None => None,
 		}
 	}
 
@@ -344,13 +362,11 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 		match self.root {
 			Some(mut id) => loop {
 				match self.node(id).child_id_opt(0) {
-					Some(child_id) => {
-						id = child_id
-					},
-					None => return Address::new(id, 0.into()) // TODO FIXME thechnically not the first
+					Some(child_id) => id = child_id,
+					None => return Address::new(id, 0.into()), // TODO FIXME thechnically not the first
 				}
 			},
-			None => Address::nowhere()
+			None => Address::nowhere(),
 		}
 	}
 
@@ -361,10 +377,10 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 				let index = node.item_count();
 				match node.child_id_opt(index) {
 					Some(child_id) => id = child_id,
-					None => return Some(Address::new(id, (index - 1).into()))
+					None => return Some(Address::new(id, (index - 1).into())),
 				}
 			},
-			None => None
+			None => None,
 		}
 	}
 
@@ -375,10 +391,10 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 				let index = node.item_count();
 				match node.child_id_opt(index) {
 					Some(child_id) => id = child_id,
-					None => return Address::new(id, index.into())
+					None => return Address::new(id, index.into()),
 				}
 			},
-			None => Address::nowhere()
+			None => Address::nowhere(),
 		}
 	}
 
@@ -393,11 +409,11 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 						Some(parent_id) => {
 							addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
 							addr.id = parent_id;
-						},
-						None => return None
+						}
+						None => return None,
 					}
 				} else {
-					return Some(addr)
+					return Some(addr);
 				}
 			}
 		}
@@ -408,12 +424,13 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 		if !addr.is_nowhere() {
 			loop {
 				let node = self.node(addr.id);
-				match node.child_id_opt(addr.offset.unwrap()) { // TODO unwrap may fail here!
+				match node.child_id_opt(addr.offset.unwrap()) {
+					// TODO unwrap may fail here!
 					Some(child_id) => {
 						addr.id = child_id;
 						addr.offset = self.node(child_id).item_count().into()
-					},
-					None => break
+					}
+					None => break,
 				}
 			}
 		}
@@ -425,33 +442,32 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 	#[inline]
 	fn previous_item_address(&self, mut addr: Address) -> Option<Address> {
 		if addr.is_nowhere() {
-			return None
+			return None;
 		}
 
 		loop {
 			let node = self.node(addr.id);
 
-			match node.child_id_opt(addr.offset.unwrap()) { // TODO unwrap may fail here.
+			match node.child_id_opt(addr.offset.unwrap()) {
+				// TODO unwrap may fail here.
 				Some(child_id) => {
 					addr.offset = self.node(child_id).item_count().into();
 					addr.id = child_id;
-				},
-				None => {
-					loop {
-						if addr.offset > 0 {
-							addr.offset.decr();
-							return Some(addr)
-						}
-
-						match self.node(addr.id).parent() {
-							Some(parent_id) => {
-								addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
-								addr.id = parent_id;
-							},
-							None => return None
-						}
-					}
 				}
+				None => loop {
+					if addr.offset > 0 {
+						addr.offset.decr();
+						return Some(addr);
+					}
+
+					match self.node(addr.id).parent() {
+						Some(parent_id) => {
+							addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
+							addr.id = parent_id;
+						}
+						None => return None,
+					}
+				},
 			}
 		}
 	}
@@ -459,7 +475,7 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 	#[inline]
 	fn previous_front_address(&self, mut addr: Address) -> Option<Address> {
 		if addr.is_nowhere() {
-			return None
+			return None;
 		}
 
 		loop {
@@ -476,24 +492,22 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 						Some(child_id) => {
 							addr.offset = (self.node(child_id).item_count()).into();
 							addr.id = child_id;
-						},
+						}
 						None => {
 							addr.offset.decr();
-							break
+							break;
 						}
 					}
-				},
-				None => {
-					match node.parent() {
-						Some(parent_id) => {
-							addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
-							addr.offset.decr();
-							addr.id = parent_id;
-							break
-						},
-						None => return None
-					}
 				}
+				None => match node.parent() {
+					Some(parent_id) => {
+						addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
+						addr.offset.decr();
+						addr.id = parent_id;
+						break;
+					}
+					None => return None,
+				},
 			}
 		}
 
@@ -504,14 +518,18 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 	#[inline]
 	fn next_item_address(&self, mut addr: Address) -> Option<Address> {
 		if addr.is_nowhere() {
-			return None
+			return None;
 		}
 
 		let item_count = self.node(addr.id).item_count();
-		if addr.offset < item_count {
-			addr.offset.incr();
-		} else if addr.offset > item_count {
-			return None
+		match addr.offset.partial_cmp(&item_count) {
+			Some(std::cmp::Ordering::Less) => {
+				addr.offset.incr();
+			}
+			Some(std::cmp::Ordering::Greater) => {
+				return None;
+			}
+			_ => (),
 		}
 
 		// let original_addr_shifted = addr;
@@ -519,27 +537,29 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 		loop {
 			let node = self.node(addr.id);
 
-			match node.child_id_opt(addr.offset.unwrap()) { // unwrap may fail here.
+			match node.child_id_opt(addr.offset.unwrap()) {
+				// unwrap may fail here.
 				Some(child_id) => {
 					addr.offset = 0.into();
 					addr.id = child_id;
-				},
+				}
 				None => {
 					loop {
 						let node = self.node(addr.id);
 
 						if addr.offset < node.item_count() {
-							return Some(addr)
+							return Some(addr);
 						}
 
 						match node.parent() {
 							Some(parent_id) => {
-								addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
+								addr.offset =
+									self.node(parent_id).child_index(addr.id).unwrap().into();
 								addr.id = parent_id;
-							},
+							}
 							None => {
 								// return Some(original_addr_shifted)
-								return None
+								return None;
 							}
 						}
 					}
@@ -551,14 +571,14 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 	#[inline]
 	fn next_back_address(&self, mut addr: Address) -> Option<Address> {
 		if addr.is_nowhere() {
-			return None
+			return None;
 		}
 
 		loop {
 			let node = self.node(addr.id);
 			let index = match addr.offset.value() {
 				Some(offset) => offset + 1,
-				None => 0
+				None => 0,
 			};
 
 			if index <= node.item_count() {
@@ -566,10 +586,10 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 					Some(child_id) => {
 						addr.offset = Offset::before();
 						addr.id = child_id;
-					},
+					}
 					None => {
 						addr.offset = index.into();
-						break
+						break;
 					}
 				}
 			} else {
@@ -577,11 +597,9 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 					Some(parent_id) => {
 						addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
 						addr.id = parent_id;
-						break
-					},
-					None => {
-						return None
+						break;
 					}
+					None => return None,
 				}
 			}
 		}
@@ -592,14 +610,18 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 	#[inline]
 	fn next_item_or_back_address(&self, mut addr: Address) -> Option<Address> {
 		if addr.is_nowhere() {
-			return None
+			return None;
 		}
 
 		let item_count = self.node(addr.id).item_count();
-		if addr.offset < item_count {
-			addr.offset.incr();
-		} else if addr.offset > item_count {
-			return None
+		match addr.offset.partial_cmp(&item_count) {
+			Some(std::cmp::Ordering::Less) => {
+				addr.offset.incr();
+			}
+			Some(std::cmp::Ordering::Greater) => {
+				return None;
+			}
+			_ => (),
 		}
 
 		let original_addr_shifted = addr;
@@ -607,50 +629,51 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 		loop {
 			let node = self.node(addr.id);
 
-			match node.child_id_opt(addr.offset.unwrap()) { // TODO unwrap may fail here.
+			match node.child_id_opt(addr.offset.unwrap()) {
+				// TODO unwrap may fail here.
 				Some(child_id) => {
 					addr.offset = 0.into();
 					addr.id = child_id;
-				},
-				None => {
-					loop {
-						let node = self.node(addr.id);
-
-						if addr.offset < node.item_count() {
-							return Some(addr)
-						}
-
-						match node.parent() {
-							Some(parent_id) => {
-								addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
-								addr.id = parent_id;
-							},
-							None => {
-								return Some(original_addr_shifted)
-							}
-						}
-					}
 				}
+				None => loop {
+					let node = self.node(addr.id);
+
+					if addr.offset < node.item_count() {
+						return Some(addr);
+					}
+
+					match node.parent() {
+						Some(parent_id) => {
+							addr.offset = self.node(parent_id).child_index(addr.id).unwrap().into();
+							addr.id = parent_id;
+						}
+						None => return Some(original_addr_shifted),
+					}
+				},
 			}
 		}
 	}
 
-	fn address_of<Q: ?Sized>(&self, key: &Q) -> Result<Address, Address> where K: Borrow<Q>, Q: Ord {
+	fn address_of<Q: ?Sized>(&self, key: &Q) -> Result<Address, Address>
+	where
+		K: Borrow<Q>,
+		Q: Ord,
+	{
 		match self.root {
 			Some(id) => self.address_in(id, key),
-			None => Err(Address::nowhere())
+			None => Err(Address::nowhere()),
 		}
 	}
 
-	fn address_in<Q: ?Sized>(&self, mut id: usize, key: &Q) -> Result<Address, Address> where K: Borrow<Q>, Q: Ord {
+	fn address_in<Q: ?Sized>(&self, mut id: usize, key: &Q) -> Result<Address, Address>
+	where
+		K: Borrow<Q>,
+		Q: Ord,
+	{
 		loop {
 			match self.node(id).offset_of(key) {
-				Ok(offset) => {
-					return Ok(Address { id, offset })
-				},
-				Err((offset, None)) => {
-					return Err(Address::new(id, offset.into()))
-				},
+				Ok(offset) => return Ok(Address { id, offset }),
+				Err((offset, None)) => return Err(Address::new(id, offset.into())),
 				Err((_, Some(child_id))) => {
 					id = child_id;
 				}
@@ -659,18 +682,27 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 	}
 
 	#[cfg(debug_assertions)]
-	fn validate(&self) where K: Ord {
-		match self.root {
-			Some(id) => {
-				self.validate_node(id, None, None, None);
-			},
-			None => ()
+	fn validate(&self)
+	where
+		K: Ord,
+	{
+		if let Some(id) = self.root {
+			self.validate_node(id, None, None, None);
 		}
 	}
 
 	/// Validate the given node and returns the depth of the node.
 	#[cfg(debug_assertions)]
-	fn validate_node(&self, id: usize, parent: Option<usize>, mut min: Option<&K>, mut max: Option<&K>) -> usize where K: Ord {
+	fn validate_node(
+		&self,
+		id: usize,
+		parent: Option<usize>,
+		mut min: Option<&K>,
+		mut max: Option<&K>,
+	) -> usize
+	where
+		K: Ord,
+	{
 		let node = self.node(id);
 		node.validate(parent, min, max);
 
@@ -693,12 +725,16 @@ impl<K, V, C: Slab<Node<K, V>>> BTreeExt<K, V> for BTreeMap<K, V, C> {
 
 		match depth {
 			Some(depth) => depth + 1,
-			None => 0
+			None => 0,
 		}
 	}
 }
 
-impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
+impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C>
+where
+	for<'r> C::ItemRef<'r>: Into<&'r Node<K, V>>,
+	for<'r> C::ItemMut<'r>: Into<&'r mut Node<K, V>>,
+{
 	#[inline]
 	fn set_len(&mut self, new_len: usize) {
 		self.len = new_len
@@ -711,11 +747,14 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 
 	#[inline]
 	fn node_mut(&mut self, id: usize) -> &mut Node<K, V> {
-		self.nodes.get_mut(id).unwrap()
+		self.nodes.get_mut(id).unwrap().into()
 	}
 
 	#[inline]
-	fn get_mut_in<'a>(&'a mut self, key: &K, mut id: usize) -> Option<&'a mut V> where K: Ord {
+	fn get_mut_in<'a>(&'a mut self, key: &K, mut id: usize) -> Option<&'a mut V>
+	where
+		K: Ord,
+	{
 		// The borrow checker is unable to predict that `*self`
 		// is not borrowed more that once at a time.
 		// That's why we need this little unsafe pointer gymnastic.
@@ -723,15 +762,11 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 		let value_ptr = loop {
 			match self.node_mut(id).get_mut(key) {
 				Ok(value_opt) => break value_opt.map(|value_ref| value_ref as *mut V),
-				Err(child_id) => {
-					id = child_id
-				}
+				Err(child_id) => id = child_id,
 			}
 		};
 
-		unsafe {
-			value_ptr.map(|ptr| &mut *ptr)
-		}
+		unsafe { value_ptr.map(|ptr| &mut *ptr) }
 	}
 
 	fn item_mut(&mut self, addr: Address) -> Option<&mut Item<K, V>> {
@@ -742,57 +777,76 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 		self.insert_exactly_at(self.leaf_address(addr), item, None)
 	}
 
-	fn insert_exactly_at(&mut self, addr: Address, item: Item<K, V>, opt_right_id: Option<usize>) -> Address {
+	fn insert_exactly_at(
+		&mut self,
+		addr: Address,
+		item: Item<K, V>,
+		opt_right_id: Option<usize>,
+	) -> Address {
 		if addr.is_nowhere() {
 			if self.is_empty() {
 				let new_root = Node::leaf(None, item);
 				let id = self.allocate_node(new_root);
 				self.root = Some(id);
 				self.len += 1;
-				Address { id, offset: 0.into() }
+				Address {
+					id,
+					offset: 0.into(),
+				}
 			} else {
 				panic!("invalid item address")
 			}
+		} else if self.is_empty() {
+			panic!("invalid item address")
 		} else {
-			if self.is_empty() {
-				panic!("invalid item address")
-			} else {
-				self.node_mut(addr.id).insert(addr.offset, item, opt_right_id);
-				let new_addr = self.rebalance(addr.id, addr);
-				self.len += 1;
-				new_addr
-			}
+			self.node_mut(addr.id)
+				.insert(addr.offset, item, opt_right_id);
+			let new_addr = self.rebalance(addr.id, addr);
+			self.len += 1;
+			new_addr
 		}
 	}
 
 	fn replace_at(&mut self, addr: Address, key: K, value: V) -> (K, V) {
-		self.node_mut(addr.id).item_mut(addr.offset).unwrap().set(key, value)
+		self.node_mut(addr.id)
+			.item_mut(addr.offset)
+			.unwrap()
+			.set(key, value)
 	}
 
 	fn replace_value_at(&mut self, addr: Address, value: V) -> V {
-		self.node_mut(addr.id).item_mut(addr.offset).unwrap().set_value(value)
+		self.node_mut(addr.id)
+			.item_mut(addr.offset)
+			.unwrap()
+			.set_value(value)
 	}
 
 	#[inline]
 	fn remove_at(&mut self, addr: Address) -> Option<(Item<K, V>, Address)> {
 		self.len -= 1;
 		match self.node_mut(addr.id).leaf_remove(addr.offset) {
-			Some(Ok(item)) => { // removed from a leaf.
+			Some(Ok(item)) => {
+				// removed from a leaf.
 				let addr = self.rebalance(addr.id, addr);
 				Some((item, addr))
-			},
-			Some(Err(left_child_id)) => { // removed from an internal node.
+			}
+			Some(Err(left_child_id)) => {
+				// removed from an internal node.
 				let new_addr = self.next_item_or_back_address(addr).unwrap();
 				let (separator, leaf_id) = self.remove_rightmost_leaf_of(left_child_id);
 				let item = self.node_mut(addr.id).replace(addr.offset, separator);
 				let addr = self.rebalance(leaf_id, new_addr);
 				Some((item, addr))
-			},
-			None => None
+			}
+			None => None,
 		}
 	}
 
-	fn update_in<T, F>(&mut self, mut id: usize, key: K, action: F) -> T where K: Ord, F: FnOnce(Option<V>) -> (Option<V>, T) {
+	fn update_in<T, F>(&mut self, mut id: usize, key: K, action: F) -> T
+	where
+		K: Ord,
+		F: FnOnce(Option<V>) -> (Option<V>, T),
+	{
 		loop {
 			match self.node(id).offset_of(&key) {
 				Ok(offset) => unsafe {
@@ -804,7 +858,7 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 						Some(new_value) => {
 							let mut new_value = MaybeUninit::new(new_value);
 							std::mem::swap(&mut new_value, item.maybe_uninit_value_mut());
-						},
+						}
 						None => {
 							let (item, _) = self.remove_at(Address::new(id, offset)).unwrap();
 							// item's value is NOT initialized here.
@@ -813,7 +867,7 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 						}
 					}
 
-					return result
+					return result;
 				},
 				Err((offset, None)) => {
 					let (opt_new_value, result) = action(None);
@@ -822,8 +876,8 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 						self.insert_exactly_at(leaf_addr, Item::new(key, new_value), None);
 					}
 
-					return result
-				},
+					return result;
+				}
 				Err((_, Some(child_id))) => {
 					id = child_id;
 				}
@@ -831,7 +885,11 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 		}
 	}
 
-	fn update_at<T, F>(&mut self, addr: Address, action: F) -> T where K: Ord, F: FnOnce(V) -> (Option<V>, T) {
+	fn update_at<T, F>(&mut self, addr: Address, action: F) -> T
+	where
+		K: Ord,
+		F: FnOnce(V) -> (Option<V>, T),
+	{
 		unsafe {
 			let mut value = MaybeUninit::uninit();
 			let item = self.node_mut(addr.id).item_mut(addr.offset).unwrap();
@@ -841,7 +899,7 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 				Some(new_value) => {
 					let mut new_value = MaybeUninit::new(new_value);
 					std::mem::swap(&mut new_value, item.maybe_uninit_value_mut());
-				},
+				}
 				None => {
 					let (item, _) = self.remove_at(addr).unwrap();
 					// item's value is NOT initialized here.
@@ -850,7 +908,7 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 				}
 			}
 
-			return result
+			result
 		}
 	}
 
@@ -860,9 +918,7 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 
 		loop {
 			match balance {
-				Balance::Balanced => {
-					break
-				},
+				Balance::Balanced => break,
 				Balance::Overflow => {
 					assert!(!self.node_mut(id).is_underflowing());
 					let (median_offset, median, right_node) = self.node_mut(id).split();
@@ -876,23 +932,29 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 
 							// new address.
 							if addr.id == id {
-								if addr.offset == median_offset {
-									addr = Address { id: parent_id, offset }
-								} else if addr.offset > median_offset {
-									addr = Address {
-										id: right_id,
-										offset: (addr.offset.unwrap() - median_offset - 1).into()
+								match addr.offset.partial_cmp(&median_offset) {
+									Some(std::cmp::Ordering::Equal) => {
+										addr = Address {
+											id: parent_id,
+											offset,
+										}
 									}
+									Some(std::cmp::Ordering::Greater) => {
+										addr = Address {
+											id: right_id,
+											offset: (addr.offset.unwrap() - median_offset - 1)
+												.into(),
+										}
+									}
+									_ => (),
 								}
-							} else if addr.id == parent_id {
-								if addr.offset >= offset {
-									addr.offset.incr()
-								}
+							} else if addr.id == parent_id && addr.offset >= offset {
+								addr.offset.incr()
 							}
 
 							id = parent_id;
 							balance = parent.balance()
-						},
+						}
 						None => {
 							let left_id = id;
 							let new_root = Node::binary(None, left_id, median, right_id);
@@ -904,28 +966,38 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 
 							// new address.
 							if addr.id == id {
-								if addr.offset == median_offset {
-									addr = Address { id: root_id, offset: 0.into() }
-								} else if addr.offset > median_offset {
-									addr = Address {
-										id: right_id,
-										offset: (addr.offset.unwrap() - median_offset - 1).into()
+								match addr.offset.partial_cmp(&median_offset) {
+									Some(std::cmp::Ordering::Equal) => {
+										addr = Address {
+											id: root_id,
+											offset: 0.into(),
+										}
 									}
+									Some(std::cmp::Ordering::Greater) => {
+										addr = Address {
+											id: right_id,
+											offset: (addr.offset.unwrap() - median_offset - 1)
+												.into(),
+										}
+									}
+									_ => (),
 								}
 							}
 
-							break
+							break;
 						}
 					};
-				},
+				}
 				Balance::Underflow(is_empty) => {
 					match self.node(id).parent() {
 						Some(parent_id) => {
 							let index = self.node(parent_id).child_index(id).unwrap();
 							// An underflow append in the child node.
 							// First we try to rebalance the tree by rotation.
-							if self.try_rotate_left(parent_id, index, &mut addr) || self.try_rotate_right(parent_id, index, &mut addr) {
-								break
+							if self.try_rotate_left(parent_id, index, &mut addr)
+								|| self.try_rotate_right(parent_id, index, &mut addr)
+							{
+								break;
 							} else {
 								// Rotation didn't work.
 								// This means that all existing child sibling have enough few elements to be merged with this child.
@@ -936,7 +1008,7 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 								// since it may underflow after the merging operation.
 								id = parent_id
 							}
-						},
+						}
 						None => {
 							// if root is empty.
 							if is_empty {
@@ -952,16 +1024,14 @@ impl<K, V, C: SlabMut<Node<K, V>>> BTreeExtMut<K, V> for BTreeMap<K, V, C> {
 											addr.id = root_id;
 											addr.offset = root.item_count().into()
 										}
-									},
-									None => {
-										addr = Address::nowhere()
 									}
+									None => addr = Address::nowhere(),
 								}
 
 								self.release_node(id);
 							}
 
-							break
+							break;
 						}
 					}
 				}
